@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import re
 from pathlib import Path
-from typing import TYPE_CHECKING, override
+from typing import override
 
 from archinstall.lib.models.device_model import (
 	BtrfsMountOption,
@@ -17,6 +17,7 @@ from archinstall.lib.models.device_model import (
 	Size,
 	Unit,
 )
+from archinstall.lib.translationhandler import tr
 from archinstall.tui.curses_menu import EditMenu, SelectMenu
 from archinstall.tui.menu_item import MenuItem, MenuItemGroup
 from archinstall.tui.result import ResultType
@@ -27,13 +28,6 @@ from ..output import FormattedOutput
 from ..utils.util import prompt_dir
 from .subvolume_menu import SubvolumeMenu
 
-if TYPE_CHECKING:
-	from collections.abc import Callable
-
-	from archinstall.lib.translationhandler import DeferredTranslation
-
-	_: Callable[[str], DeferredTranslation]
-
 
 class FreeSpace:
 	def __init__(self, start: Size, end: Size) -> None:
@@ -81,7 +75,7 @@ class PartitioningList(ListManager[DiskSegment]):
 	def __init__(
 		self,
 		device_mod: DeviceModification,
-		partition_table: PartitionTable
+		partition_table: PartitionTable,
 	) -> None:
 		device = device_mod.device
 
@@ -91,24 +85,28 @@ class PartitioningList(ListManager[DiskSegment]):
 		self._using_gpt = device_mod.using_gpt(partition_table)
 
 		self._actions = {
-			'suggest_partition_layout': str(_('Suggest partition layout')),
-			'remove_added_partitions': str(_('Remove all newly added partitions')),
-			'assign_mountpoint': str(_('Assign mountpoint')),
-			'mark_formatting': str(_('Mark/Unmark to be formatted (wipes data)')),
-			'mark_bootable': str(_('Mark/Unmark as bootable')),
+			'suggest_partition_layout': tr('Suggest partition layout'),
+			'remove_added_partitions': tr('Remove all newly added partitions'),
+			'assign_mountpoint': tr('Assign mountpoint'),
+			'mark_formatting': tr('Mark/Unmark to be formatted (wipes data)'),
+			'mark_bootable': tr('Mark/Unmark as bootable'),
 		}
 		if self._using_gpt:
-			self._actions.update({
-				'mark_esp': str(_('Mark/Unmark as ESP')),
-				'mark_xbootldr': str(_('Mark/Unmark as XBOOTLDR'))
-			})
-		self._actions.update({
-			'set_filesystem': str(_('Change filesystem')),
-			'btrfs_mark_compressed': str(_('Mark/Unmark as compressed')),  # btrfs only
-			'btrfs_mark_nodatacow': str(_('Mark/Unmark as nodatacow')),  # btrfs only
-			'btrfs_set_subvolumes': str(_('Set subvolumes')),  # btrfs only
-			'delete_partition': str(_('Delete partition'))
-		})
+			self._actions.update(
+				{
+					'mark_esp': tr('Mark/Unmark as ESP'),
+					'mark_xbootldr': tr('Mark/Unmark as XBOOTLDR'),
+				}
+			)
+		self._actions.update(
+			{
+				'set_filesystem': tr('Change filesystem'),
+				'btrfs_mark_compressed': tr('Mark/Unmark as compressed'),  # btrfs only
+				'btrfs_mark_nodatacow': tr('Mark/Unmark as nodatacow'),  # btrfs only
+				'btrfs_set_subvolumes': tr('Set subvolumes'),  # btrfs only
+				'delete_partition': tr('Delete partition'),
+			}
+		)
 
 		device_partitions = []
 
@@ -116,13 +114,13 @@ class PartitioningList(ListManager[DiskSegment]):
 			# we'll display the existing partitions of the device
 			for partition in device.partition_infos:
 				device_partitions.append(
-					PartitionModification.from_existing_partition(partition)
+					PartitionModification.from_existing_partition(partition),
 				)
 		else:
 			device_partitions = device_mod.partitions
 
-		prompt = str(_('Partition management: {}')).format(device.device_info.path) + '\n'
-		prompt += str(_('Total length: {}')).format(device.device_info.total_size.format_size(Unit.MiB))
+		prompt = tr('Partition management: {}').format(device.device_info.path) + '\n'
+		prompt += tr('Total length: {}').format(device.device_info.total_size.format_size(Unit.MiB))
 		self._info = prompt + '\n'
 
 		display_actions = list(self._actions.values())
@@ -130,11 +128,11 @@ class PartitioningList(ListManager[DiskSegment]):
 			self.as_segments(device_partitions),
 			display_actions[:1],
 			display_actions[2:],
-			self._info + self.wipe_str()
+			self._info + self.wipe_str(),
 		)
 
 	def wipe_str(self) -> str:
-		return '{}: {}'.format(str(_('Wipe')), self._wipe)
+		return '{}: {}'.format(tr('Wipe'), self._wipe)
 
 	def as_segments(self, device_partitions: list[PartitionModification]) -> list[DiskSegment]:
 		end = self._device.device_info.total_size
@@ -157,9 +155,9 @@ class PartitioningList(ListManager[DiskSegment]):
 			return segments
 
 		first_part_index, first_partition = next(
-			(i, disk_segment) for i, disk_segment in enumerate(segments)
-			if isinstance(disk_segment.segment, PartitionModification)
-			and not disk_segment.segment.is_delete()
+			(i, disk_segment)
+			for i, disk_segment in enumerate(segments)
+			if isinstance(disk_segment.segment, PartitionModification) and not disk_segment.segment.is_delete()
 		)
 
 		prev_partition = first_partition
@@ -193,10 +191,7 @@ class PartitioningList(ListManager[DiskSegment]):
 
 	@staticmethod
 	def get_part_mods(disk_segments: list[DiskSegment]) -> list[PartitionModification]:
-		return [
-			s.segment for s in disk_segments
-			if isinstance(s.segment, PartitionModification)
-		]
+		return [s.segment for s in disk_segments if isinstance(s.segment, PartitionModification)]
 
 	def get_device_mod(self) -> DeviceModification:
 		disk_segments = super().run()
@@ -215,9 +210,9 @@ class PartitioningList(ListManager[DiskSegment]):
 	def selected_action_display(self, selection: DiskSegment) -> str:
 		if isinstance(selection.segment, PartitionModification):
 			if selection.segment.status == ModificationStatus.Create:
-				return str(_('Partition - New'))
+				return tr('Partition - New')
 			elif selection.segment.is_delete() and selection.segment.dev_path:
-				title = str(_('Partition')) + '\n\n'
+				title = tr('Partition') + '\n\n'
 				title += 'status: delete\n'
 				title += f'device: {selection.segment.dev_path}\n'
 				for part in self._device.partition_infos:
@@ -246,17 +241,17 @@ class PartitioningList(ListManager[DiskSegment]):
 				#     how do we know it was the original one?
 				not_filter += [
 					self._actions['set_filesystem'],
-					self._actions['mark_bootable']
+					self._actions['mark_bootable'],
 				]
 				if self._using_gpt:
 					not_filter += [
 						self._actions['mark_esp'],
-						self._actions['mark_xbootldr']
+						self._actions['mark_xbootldr'],
 					]
 				not_filter += [
 					self._actions['btrfs_mark_compressed'],
 					self._actions['btrfs_mark_nodatacow'],
-					self._actions['btrfs_set_subvolumes']
+					self._actions['btrfs_set_subvolumes'],
 				]
 
 			# non btrfs partitions shouldn't get btrfs options
@@ -264,7 +259,7 @@ class PartitioningList(ListManager[DiskSegment]):
 				not_filter += [
 					self._actions['btrfs_mark_compressed'],
 					self._actions['btrfs_mark_nodatacow'],
-					self._actions['btrfs_set_subvolumes']
+					self._actions['btrfs_set_subvolumes'],
 				]
 			else:
 				not_filter += [self._actions['assign_mountpoint']]
@@ -276,7 +271,7 @@ class PartitioningList(ListManager[DiskSegment]):
 		self,
 		action: str,
 		entry: DiskSegment | None,
-		data: list[DiskSegment]
+		data: list[DiskSegment],
 	) -> list[DiskSegment]:
 		if not entry:
 			action_key = [k for k, v in self._actions.items() if v == action][0]
@@ -290,11 +285,7 @@ class PartitioningList(ListManager[DiskSegment]):
 						self._prompt = self._info + self.wipe_str()
 				case 'remove_added_partitions':
 					if self._reset_confirmation():
-						data = [
-							s for s in data
-							if isinstance(s.segment, PartitionModification)
-							and s.segment.is_exists_or_modify()
-						]
+						data = [s for s in data if isinstance(s.segment, PartitionModification) and s.segment.is_exists_or_modify()]
 		elif isinstance(entry.segment, PartitionModification):
 			partition = entry.segment
 			action_key = [k for k, v in self._actions.items() if v == action][0]
@@ -332,17 +323,17 @@ class PartitioningList(ListManager[DiskSegment]):
 						partition.invert_flag(PartitionFlag.XBOOTLDR)
 				case 'set_filesystem':
 					fs_type = self._prompt_partition_fs_type()
-					if fs_type:
-						if partition.is_swap():
-							partition.invert_flag(PartitionFlag.SWAP)
-						partition.fs_type = fs_type
-						if partition.is_swap():
-							partition.mountpoint = None
-							partition.flags = []
-							partition.set_flag(PartitionFlag.SWAP)
-						# btrfs subvolumes will define mountpoints
-						if fs_type == FilesystemType.Btrfs:
-							partition.mountpoint = None
+
+					if partition.is_swap():
+						partition.invert_flag(PartitionFlag.SWAP)
+					partition.fs_type = fs_type
+					if partition.is_swap():
+						partition.mountpoint = None
+						partition.flags = []
+						partition.set_flag(PartitionFlag.SWAP)
+					# btrfs subvolumes will define mountpoints
+					if fs_type == FilesystemType.Btrfs:
+						partition.mountpoint = None
 				case 'btrfs_mark_compressed':
 					self._toggle_mount_option(partition, BtrfsMountOption.compress)
 				case 'btrfs_mark_nodatacow':
@@ -362,47 +353,35 @@ class PartitioningList(ListManager[DiskSegment]):
 	def _delete_partition(
 		self,
 		entry: PartitionModification,
-		data: list[DiskSegment]
+		data: list[DiskSegment],
 	) -> list[DiskSegment]:
 		if entry.is_exists_or_modify():
 			entry.status = ModificationStatus.Delete
 			part_mods = self.get_part_mods(data)
 		else:
-			part_mods = [
-				d.segment for d in data
-				if isinstance(d.segment, PartitionModification)
-				and d.segment != entry
-			]
+			part_mods = [d.segment for d in data if isinstance(d.segment, PartitionModification) and d.segment != entry]
 
 		return self.as_segments(part_mods)
 
 	def _toggle_mount_option(
 		self,
 		partition: PartitionModification,
-		option: BtrfsMountOption
+		option: BtrfsMountOption,
 	) -> None:
 		if option.value not in partition.mount_options:
 			if option == BtrfsMountOption.compress:
-				partition.mount_options = [
-					o for o in partition.mount_options
-					if o != BtrfsMountOption.nodatacow.value
-				]
+				partition.mount_options = [o for o in partition.mount_options if o != BtrfsMountOption.nodatacow.value]
 
-			partition.mount_options = [
-				o for o in partition.mount_options
-				if not o.startswith(BtrfsMountOption.compress.name)
-			]
+			partition.mount_options = [o for o in partition.mount_options if not o.startswith(BtrfsMountOption.compress.name)]
 
 			partition.mount_options.append(option.value)
 		else:
-			partition.mount_options = [
-				o for o in partition.mount_options if o != option.value
-			]
+			partition.mount_options = [o for o in partition.mount_options if o != option.value]
 
 	def _set_btrfs_subvolumes(self, partition: PartitionModification) -> None:
 		partition.btrfs_subvols = SubvolumeMenu(
 			partition.btrfs_subvols,
-			None
+			None,
 		).run()
 
 	def _prompt_formatting(self, partition: PartitionModification) -> None:
@@ -417,7 +396,7 @@ class PartitioningList(ListManager[DiskSegment]):
 		# without asking the user which inner-filesystem they want to use. Since the flag 'encrypted' = True is already set,
 		# it's safe to change the filesystem for this partition.
 		if partition.fs_type == FilesystemType.Crypto_luks:
-			prompt = str(_('This partition is currently encrypted, to format it a filesystem has to be specified')) + '\n'
+			prompt = tr('This partition is currently encrypted, to format it a filesystem has to be specified') + '\n'
 			fs_type = self._prompt_partition_fs_type(prompt)
 			partition.fs_type = fs_type
 
@@ -425,8 +404,8 @@ class PartitioningList(ListManager[DiskSegment]):
 				partition.mountpoint = None
 
 	def _prompt_mountpoint(self) -> Path:
-		header = str(_('Partition mount-points are relative to inside the installation, the boot would be /boot as an example.')) + '\n'
-		prompt = str(_('Mountpoint'))
+		header = tr('Partition mount-points are relative to inside the installation, the boot would be /boot as an example.') + '\n'
+		prompt = tr('Mountpoint')
 
 		mountpoint = prompt_dir(prompt, header, validate=False, allow_skip=False)
 		assert mountpoint
@@ -442,8 +421,8 @@ class PartitioningList(ListManager[DiskSegment]):
 			group,
 			header=prompt,
 			alignment=Alignment.CENTER,
-			frame=FrameProperties.min(str(_('Filesystem'))),
-			allow_skip=False
+			frame=FrameProperties.min(tr('Filesystem')),
+			allow_skip=False,
 		).run()
 
 		match result.type_:
@@ -456,7 +435,7 @@ class PartitioningList(ListManager[DiskSegment]):
 		self,
 		sector_size: SectorSize,
 		max_size: Size,
-		text: str
+		text: str,
 	) -> Size | None:
 		match = re.match(r'([0-9]+)([a-zA-Z|%]*)', text, re.I)
 
@@ -485,35 +464,38 @@ class PartitioningList(ListManager[DiskSegment]):
 		return size
 
 	def _prompt_size(self, free_space: FreeSpace) -> Size:
-		def validate(value: str) -> str | None:
+		def validate(value: str | None) -> str | None:
+			if not value:
+				return None
+
 			size = self._validate_value(sector_size, max_size, value)
 			if not size:
-				return str(_('Invalid size'))
+				return tr('Invalid size')
 			return None
 
 		device_info = self._device.device_info
 		sector_size = device_info.sector_size
 
-		text = str(_('Selected free space segment on device {}:')).format(device_info.path) + '\n\n'
+		text = tr('Selected free space segment on device {}:').format(device_info.path) + '\n\n'
 		free_space_table = FormattedOutput.as_table([free_space])
 		prompt = text + free_space_table + '\n'
 
 		max_sectors = free_space.length.format_size(Unit.sectors, sector_size)
 		max_bytes = free_space.length.format_size(Unit.B)
 
-		prompt += str(_('Size: {} / {}')).format(max_sectors, max_bytes) + '\n\n'
-		prompt += str(_('All entered values can be suffixed with a unit: %, B, KB, KiB, MB, MiB...')) + '\n'
-		prompt += str(_('If no unit is provided, the value is interpreted as sectors')) + '\n'
+		prompt += tr('Size: {} / {}').format(max_sectors, max_bytes) + '\n\n'
+		prompt += tr('All entered values can be suffixed with a unit: %, B, KB, KiB, MB, MiB...') + '\n'
+		prompt += tr('If no unit is provided, the value is interpreted as sectors') + '\n'
 
 		max_size = free_space.length
 
-		title = str(_('Size (default: {}): ')).format(max_size.format_highest())
+		title = tr('Size (default: {}): ').format(max_size.format_highest())
 
 		result = EditMenu(
 			title,
 			header=f'{prompt}\b',
 			allow_skip=True,
-			validator=validate
+			validator=validate,
 		).input()
 
 		size: Size | None = None
@@ -547,7 +529,7 @@ class PartitioningList(ListManager[DiskSegment]):
 			start=free_space.start,
 			length=length,
 			fs_type=fs_type,
-			mountpoint=mountpoint
+			mountpoint=mountpoint,
 		)
 
 		if partition.mountpoint == Path('/boot'):
@@ -562,7 +544,7 @@ class PartitioningList(ListManager[DiskSegment]):
 		return partition
 
 	def _reset_confirmation(self) -> bool:
-		prompt = str(_('This will remove all newly added partitions, continue?')) + '\n'
+		prompt = tr('This will remove all newly added partitions, continue?') + '\n'
 
 		result = SelectMenu[bool](
 			MenuItemGroup.yes_no(),
@@ -571,14 +553,14 @@ class PartitioningList(ListManager[DiskSegment]):
 			orientation=Orientation.HORIZONTAL,
 			columns=2,
 			reset_warning_msg=prompt,
-			allow_skip=False
+			allow_skip=False,
 		).run()
 
 		return result.item() == MenuItem.yes()
 
 	def _suggest_partition_layout(
 		self,
-		data: list[PartitionModification]
+		data: list[PartitionModification],
 	) -> DeviceModification | None:
 		# if modifications have been done already, inform the user
 		# that this operation will erase those modifications
@@ -593,7 +575,7 @@ class PartitioningList(ListManager[DiskSegment]):
 
 def manual_partitioning(
 	device_mod: DeviceModification,
-	partition_table: PartitionTable
+	partition_table: PartitionTable,
 ) -> DeviceModification | None:
 	menu_list = PartitioningList(device_mod, partition_table)
 	mod = menu_list.get_device_mod()
