@@ -1,46 +1,50 @@
 from __future__ import annotations
 
+from enum import Enum
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import assert_never
 
 from archinstall.lib.models.packages import Repository
 from archinstall.lib.packages.packages import list_available_packages
+from archinstall.lib.translationhandler import tr
 from archinstall.tui.curses_menu import EditMenu, SelectMenu, Tui
 from archinstall.tui.menu_item import MenuItem, MenuItemGroup
-from archinstall.tui.types import Alignment, FrameProperties, Orientation, PreviewStyle, ResultType
+from archinstall.tui.result import ResultType
+from archinstall.tui.types import Alignment, FrameProperties, Orientation, PreviewStyle
 
 from ..locale.utils import list_timezones
 from ..models.audio_configuration import Audio, AudioConfiguration
-from ..models.packages import AvailablePackage
+from ..models.packages import AvailablePackage, PackageGroup
 from ..output import warn
 from ..translationhandler import Language
 
-if TYPE_CHECKING:
-	from collections.abc import Callable
 
-	from archinstall.lib.translationhandler import DeferredTranslation
-
-	_: Callable[[str], DeferredTranslation]
+class PostInstallationAction(Enum):
+	EXIT = tr('Exit archinstall')
+	REBOOT = tr('Reboot system')
+	CHROOT = tr('chroot into installation for post-installation configurations')
 
 
 def ask_ntp(preset: bool = True) -> bool:
-	header = str(_('Would you like to use automatic time synchronization (NTP) with the default time servers?\n')) + '\n'
-	header += str(_(
-		'Hardware time and other post-configuration steps might be required in order for NTP to work.\n'
-		'For more information, please check the Arch wiki'
-	)) + '\n'
+	header = tr('Would you like to use automatic time synchronization (NTP) with the default time servers?\n') + '\n'
+	header += (
+		tr(
+			'Hardware time and other post-configuration steps might be required in order for NTP to work.\nFor more information, please check the Arch wiki',
+		)
+		+ '\n'
+	)
 
 	preset_val = MenuItem.yes() if preset else MenuItem.no()
 	group = MenuItemGroup.yes_no()
 	group.focus_item = preset_val
 
-	result = SelectMenu(
+	result = SelectMenu[bool](
 		group,
 		header=header,
 		allow_skip=True,
 		alignment=Alignment.CENTER,
 		columns=2,
-		orientation=Orientation.HORIZONTAL
+		orientation=Orientation.HORIZONTAL,
 	).run()
 
 	match result.type_:
@@ -54,10 +58,10 @@ def ask_ntp(preset: bool = True) -> bool:
 
 def ask_hostname(preset: str | None = None) -> str | None:
 	result = EditMenu(
-		str(_('Hostname')),
+		tr('Hostname'),
 		alignment=Alignment.CENTER,
 		allow_skip=True,
-		default_text=preset
+		default_text=preset,
 	).input()
 
 	match result.type_:
@@ -81,11 +85,11 @@ def ask_for_a_timezone(preset: str | None = None) -> str | None:
 	group.set_selected_by_value(preset)
 	group.set_default_by_value(default)
 
-	result = SelectMenu(
+	result = SelectMenu[str](
 		group,
 		allow_reset=True,
 		allow_skip=True,
-		frame=FrameProperties.min(str(_('Timezone'))),
+		frame=FrameProperties.min(tr('Timezone')),
 		alignment=Alignment.CENTER,
 	).run()
 
@@ -105,11 +109,11 @@ def ask_for_audio_selection(preset: AudioConfiguration | None = None) -> AudioCo
 	if preset:
 		group.set_focus_by_value(preset.audio)
 
-	result = SelectMenu(
+	result = SelectMenu[Audio](
 		group,
 		allow_skip=True,
 		alignment=Alignment.CENTER,
-		frame=FrameProperties.min(str(_('Audio')))
+		frame=FrameProperties.min(tr('Audio')),
 	).run()
 
 	match result.type_:
@@ -129,8 +133,7 @@ def select_language(preset: str | None = None) -> str | None:
 	# raise Deprecated("select_language() has been deprecated, use select_kb_layout() instead.")
 
 	# No need to translate this i feel, as it's a short lived message.
-	warn(
-		"select_language() is deprecated, use select_kb_layout() instead. select_language() will be removed in a future version")
+	warn('select_language() is deprecated, use select_kb_layout() instead. select_language() will be removed in a future version')
 
 	return select_kb_layout(preset)
 
@@ -148,13 +151,13 @@ def select_archinstall_language(languages: list[Language], preset: Language) ->
 	title += 'All available fonts can be found in "/usr/share/kbd/consolefonts"\n'
 	title += 'e.g. setfont LatGrkCyr-8x16 (to display latin/greek/cyrillic characters)\n'
 
-	result = SelectMenu(
+	result = SelectMenu[Language](
 		group,
 		header=title,
 		allow_skip=True,
 		allow_reset=False,
 		alignment=Alignment.CENTER,
-		frame=FrameProperties.min(header=str(_('Select language')))
+		frame=FrameProperties.min(header=tr('Select language')),
 	).run()
 
 	match result.type_:
@@ -168,36 +171,54 @@ def select_archinstall_language(languages: list[Language], preset: Language) ->
 
 def ask_additional_packages_to_install(
 	preset: list[str] = [],
-	repositories: set[Repository] = set()
+	repositories: set[Repository] = set(),
 ) -> list[str]:
-	Tui.print(str(_('Loading packages...')), clear_screen=True)
-
 	repositories |= {Repository.Core, Repository.Extra}
+
+	respos_text = ', '.join([r.value for r in repositories])
+	output = tr('Repositories: {}').format(respos_text) + '\n'
+
+	output += tr('Loading packages...')
+	Tui.print(output, clear_screen=True)
+
 	packages = list_available_packages(tuple(repositories))
+	package_groups = PackageGroup.from_available_packages(packages)
 
 	# Additional packages (with some light weight error handling for invalid package names)
-	header = str(_('Only packages such as base, base-devel, linux, linux-firmware, efibootmgr and optional profile packages are installed.')) + '\n'
-	header += str(_('Select any packages from the below list that should be installed additionally')) + '\n'
+	header = tr('Only packages such as base, base-devel, linux, linux-firmware, efibootmgr and optional profile packages are installed.') + '\n'
+	header += tr('Select any packages from the below list that should be installed additionally') + '\n'
 
 	# there are over 15k packages so this needs to be quick
-	preset_packages = []
+	preset_packages: list[AvailablePackage | PackageGroup] = []
 	for p in preset:
 		if p in packages:
 			preset_packages.append(packages[p])
+		elif p in package_groups:
+			preset_packages.append(package_groups[p])
 
 	items = [
 		MenuItem(
 			name,
 			value=pkg,
-			preview_action=lambda x: x.value.info()
-		) for name,
-		pkg in packages.items()
+			preview_action=lambda x: x.value.info(),
+		)
+		for name, pkg in packages.items()
 	]
-	group = MenuItemGroup(items, sort_items=True)
-	group.set_selected_by_value(preset_packages)
 
-	result = SelectMenu(
-		group,
+	items += [
+		MenuItem(
+			name,
+			value=group,
+			preview_action=lambda x: x.value.info(),
+		)
+		for name, group in package_groups.items()
+	]
+
+	menu_group = MenuItemGroup(items, sort_items=True)
+	menu_group.set_selected_by_value(preset_packages)
+
+	result = SelectMenu[AvailablePackage | PackageGroup](
+		menu_group,
 		header=header,
 		alignment=Alignment.LEFT,
 		allow_reset=True,
@@ -205,7 +226,7 @@ def ask_additional_packages_to_install(
 		multi=True,
 		preview_frame=FrameProperties.max('Package info'),
 		preview_style=PreviewStyle.RIGHT,
-		preview_size='auto'
+		preview_size='auto',
 	).run()
 
 	match result.type_:
@@ -214,35 +235,36 @@ def ask_additional_packages_to_install(
 		case ResultType.Reset:
 			return []
 		case ResultType.Selection:
-			selected_pacakges: list[AvailablePackage] = result.get_values()
+			selected_pacakges = result.get_values()
 			return [pkg.name for pkg in selected_pacakges]
 
 
 def add_number_of_parallel_downloads(preset: int | None = None) -> int | None:
 	max_recommended = 5
 
-	header = str(_('This option enables the number of parallel downloads that can occur during package downloads')) + '\n'
-	header += str(_('Enter the number of parallel downloads to be enabled.\n\nNote:\n'))
-	header += str(_(' - Maximum recommended value : {} ( Allows {} parallel downloads at a time )')).format(max_recommended, max_recommended) + '\n'
-	header += str(_(' - Disable/Default : 0 ( Disables parallel downloading, allows only 1 download at a time )\n'))
+	header = tr('This option enables the number of parallel downloads that can occur during package downloads') + '\n'
+	header += tr('Enter the number of parallel downloads to be enabled.\n\nNote:\n')
+	header += tr(' - Maximum recommended value : {} ( Allows {} parallel downloads at a time )').format(max_recommended, max_recommended) + '\n'
+	header += tr(' - Disable/Default : 0 ( Disables parallel downloading, allows only 1 download at a time )\n')
 
-	def validator(s: str) -> str | None:
-		try:
-			value = int(s)
-			if value >= 0:
-				return None
-		except Exception:
-			pass
+	def validator(s: str | None) -> str | None:
+		if s is not None:
+			try:
+				value = int(s)
+				if value >= 0:
+					return None
+			except Exception:
+				pass
 
-		return str(_('Invalid download number'))
+		return tr('Invalid download number')
 
 	result = EditMenu(
-		str(_('Number downloads')),
+		tr('Number downloads'),
 		header=header,
 		allow_skip=True,
 		allow_reset=True,
 		validator=validator,
-		default_text=str(preset) if preset is not None else None
+		default_text=str(preset) if preset is not None else None,
 	).input()
 
 	match result.type_:
@@ -252,47 +274,55 @@ def add_number_of_parallel_downloads(preset: int | None = None) -> int | None:
 			return 0
 		case ResultType.Selection:
 			downloads: int = int(result.text())
+		case _:
+			assert_never(result.type_)
 
-	pacman_conf_path = Path("/etc/pacman.conf")
+	pacman_conf_path = Path('/etc/pacman.conf')
 	with pacman_conf_path.open() as f:
-		pacman_conf = f.read().split("\n")
+		pacman_conf = f.read().split('\n')
 
-	with pacman_conf_path.open("w") as fwrite:
+	with pacman_conf_path.open('w') as fwrite:
 		for line in pacman_conf:
-			if "ParallelDownloads" in line:
-				fwrite.write(f"ParallelDownloads = {downloads}\n")
+			if 'ParallelDownloads' in line:
+				fwrite.write(f'ParallelDownloads = {downloads}\n')
 			else:
-				fwrite.write(f"{line}\n")
+				fwrite.write(f'{line}\n')
 
 	return downloads
 
 
-def ask_chroot() -> bool:
-	prompt = str(_('Would you like to chroot into the newly created installation and perform post-installation configuration?')) + '\n'
-	group = MenuItemGroup.yes_no()
+def ask_post_installation() -> PostInstallationAction:
+	header = tr('Installation completed') + '\n\n'
+	header += tr('What would you like to do next?') + '\n'
 
-	result = SelectMenu(
+	items = [MenuItem(action.value, value=action) for action in PostInstallationAction]
+	group = MenuItemGroup(items)
+
+	result = SelectMenu[PostInstallationAction](
 		group,
-		header=prompt,
+		header=header,
+		allow_skip=False,
 		alignment=Alignment.CENTER,
-		columns=2,
-		orientation=Orientation.HORIZONTAL,
 	).run()
 
-	return result.item() == MenuItem.yes()
+	match result.type_:
+		case ResultType.Selection:
+			return result.get_value()
+		case _:
+			raise ValueError('Post installation action not handled')
 
 
 def ask_abort() -> None:
-	prompt = str(_('Do you really want to abort?')) + '\n'
+	prompt = tr('Do you really want to abort?') + '\n'
 	group = MenuItemGroup.yes_no()
 
-	result = SelectMenu(
+	result = SelectMenu[bool](
 		group,
 		header=prompt,
 		allow_skip=False,
 		alignment=Alignment.CENTER,
 		columns=2,
-		orientation=Orientation.HORIZONTAL
+		orientation=Orientation.HORIZONTAL,
 	).run()
 
 	if result.item() == MenuItem.yes():
