@@ -1,13 +1,16 @@
 from __future__ import annotations
 
 import shlex
-import time
 from dataclasses import dataclass
 from pathlib import Path
+from subprocess import CalledProcessError
+from types import TracebackType
+
+from archinstall.lib.disk.utils import get_lsblk_info, umount
 
-from . import disk
 from .exceptions import DiskError, SysCallError
-from .general import SysCommand, SysCommandWorker, generate_password
+from .general import SysCommand, SysCommandWorker, generate_password, run
+from .models.users import Password
 from .output import debug, info
 
 
@@ -15,7 +18,7 @@ from .output import debug, info
 class Luks2:
 	luks_dev_path: Path
 	mapper_name: str | None = None
-	password: str | None = None
+	password: Password | None = None
 	key_file: Path | None = None
 	auto_unmount: bool = False
 
@@ -45,7 +48,7 @@ class Luks2:
 	def __enter__(self) -> None:
 		self.unlock(self.key_file)
 
-	def __exit__(self, *args: str, **kwargs: str) -> None:
+	def __exit__(self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None) -> None:
 		if self.auto_unmount:
 			self.lock()
 
@@ -56,52 +59,59 @@ class Luks2:
 		if isinstance(self.password, bytes):
 			return self.password
 		else:
-			return bytes(self.password, 'UTF-8')
+			return bytes(self.password.plaintext, 'UTF-8')
+
+	def _get_passphrase_args(
+		self,
+		key_file: Path | None = None,
+	) -> tuple[list[str], bytes | None]:
+		key_file = key_file or self.key_file
 
-	def _get_key_file(self, key_file: Path | None = None) -> Path:
 		if key_file:
-			return key_file
+			return ['--key-file', str(key_file)], None
 
-		if self.key_file:
-			return self.key_file
-
-		default_key_file = Path(f'/tmp/{self.luks_dev_path.name}.disk_pw')
-		default_key_file.write_bytes(self._password_bytes())
-		return default_key_file
+		return [], self._password_bytes()
 
 	def encrypt(
 		self,
 		key_size: int = 512,
 		hash_type: str = 'sha512',
 		iter_time: int = 10000,
-		key_file: Path | None = None
-	) -> Path:
+		key_file: Path | None = None,
+	) -> Path | None:
 		debug(f'Luks2 encrypting: {self.luks_dev_path}')
 
-		key_file = self._get_key_file(key_file)
+		key_file_arg, passphrase = self._get_passphrase_args(key_file)
 
-		cryptsetup_args = shlex.join([
+		cmd = [
 			'cryptsetup',
 			'--batch-mode',
 			'--verbose',
-			'--type', 'luks2',
-			'--pbkdf', 'argon2id',
-			'--hash', hash_type,
-			'--key-size', str(key_size),
-			'--iter-time', str(iter_time),
-			'--key-file', str(key_file),
+			'--type',
+			'luks2',
+			'--pbkdf',
+			'argon2id',
+			'--hash',
+			hash_type,
+			'--key-size',
+			str(key_size),
+			'--iter-time',
+			str(iter_time),
+			*key_file_arg,
 			'--use-urandom',
-			'luksFormat', str(self.luks_dev_path),
-		])
+			'luksFormat',
+			str(self.luks_dev_path),
+		]
 
-		debug(f'cryptsetup format: {cryptsetup_args}')
+		debug(f'cryptsetup format: {shlex.join(cmd)}')
 
 		try:
-			result = SysCommand(cryptsetup_args).decode()
-		except SysCallError as err:
-			raise DiskError(f'Could not encrypt volume "{self.luks_dev_path}": {err}')
+			result = run(cmd, input_data=passphrase)
+		except CalledProcessError as err:
+			output = err.stdout.decode().rstrip()
+			raise DiskError(f'Could not encrypt volume "{self.luks_dev_path}": {output}')
 
-		debug(f'cryptsetup luksFormat output: {result}')
+		debug(f'cryptsetup luksFormat output: {result.stdout.decode().rstrip()}')
 
 		self.key_file = key_file
 
@@ -117,7 +127,7 @@ class Luks2:
 			raise err
 
 	def is_unlocked(self) -> bool:
-		return self.mapper_name is not None and Path(f'/dev/mapper/{self.mapper_name}').exists()
+		return (mapper_dev := self.mapper_dev) is not None and mapper_dev.is_symlink()
 
 	def unlock(self, key_file: Path | None = None) -> None:
 		"""
@@ -132,42 +142,42 @@ class Luks2:
 		if not self.mapper_name:
 			raise ValueError('mapper name missing')
 
-		key_file = self._get_key_file(key_file)
+		key_file_arg, passphrase = self._get_passphrase_args(key_file)
 
-		wait_timer = time.time()
-		while Path(self.luks_dev_path).exists() is False and time.time() - wait_timer < 10:
-			time.sleep(0.025)
+		cmd = [
+			'cryptsetup',
+			'open',
+			str(self.luks_dev_path),
+			str(self.mapper_name),
+			*key_file_arg,
+			'--type',
+			'luks2',
+		]
 
-		result = SysCommand(
-			'cryptsetup open '
-			f'{self.luks_dev_path} '
-			f'{self.mapper_name} '
-			f'--key-file {key_file} '
-			f'--type luks2'
-		).decode()
+		result = run(cmd, input_data=passphrase)
 
-		debug(f'cryptsetup open output: {result}')
+		debug(f'cryptsetup open output: {result.stdout.decode().rstrip()}')
 
-		if not self.mapper_dev or not self.mapper_dev.is_symlink():
+		if not self.is_unlocked():
 			raise DiskError(f'Failed to open luks2 device: {self.luks_dev_path}')
 
 	def lock(self) -> None:
-		disk.device_handler.umount(self.luks_dev_path)
+		umount(self.luks_dev_path)
 
 		# Get crypt-information about the device by doing a reverse lookup starting with the partition path
 		# For instance: /dev/sda
-		lsblk_info = disk.get_lsblk_info(self.luks_dev_path)
+		lsblk_info = get_lsblk_info(self.luks_dev_path)
 
 		# For each child (sub-partition/sub-device)
 		for child in lsblk_info.children:
 			# Unmount the child location
 			for mountpoint in child.mountpoints:
 				debug(f'Unmounting {mountpoint}')
-				disk.device_handler.umount(mountpoint, recursive=True)
+				umount(mountpoint, recursive=True)
 
 			# And close it if possible.
-			debug(f"Closing crypt device {child.name}")
-			SysCommand(f"cryptsetup close {child.name}")
+			debug(f'Closing crypt device {child.name}')
+			SysCommand(f'cryptsetup close {child.name}')
 
 	def create_keyfile(self, target_path: Path, override: bool = False) -> None:
 		"""
@@ -197,7 +207,7 @@ class Luks2:
 		key_file.chmod(0o400)
 
 		self._add_key(key_file)
-		self._crypttab(crypttab_path, kf_path, options=["luks", "key-slot=1"])
+		self._crypttab(crypttab_path, kf_path, options=['luks', 'key-slot=1'])
 
 	def _add_key(self, key_file: Path) -> None:
 		debug(f'Adding additional key-file {key_file}')
@@ -218,12 +228,12 @@ class Luks2:
 		self,
 		crypttab_path: Path,
 		key_file: Path,
-		options: list[str]
+		options: list[str],
 	) -> None:
 		debug(f'Adding crypttab entry for key {key_file}')
 
 		with open(crypttab_path, 'a') as crypttab:
 			opt = ','.join(options)
 			uuid = self._get_luks_uuid()
-			row = f"{self.mapper_name} UUID={uuid} {key_file} {opt}\n"
+			row = f'{self.mapper_name} UUID={uuid} {key_file} {opt}\n'
 			crypttab.write(row)
