@@ -1,76 +1,72 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any
+from typing import override
 
 from archinstall.default_profiles.profile import GreeterType, Profile
-from archinstall.tui import Alignment, FrameProperties, MenuItem, MenuItemGroup, Orientation, ResultType, SelectMenu
+from archinstall.lib.translationhandler import tr
+from archinstall.tui.curses_menu import SelectMenu
+from archinstall.tui.menu_item import MenuItem, MenuItemGroup
+from archinstall.tui.result import ResultType
+from archinstall.tui.types import Alignment, FrameProperties, Orientation
 
 from ..hardware import GfxDriver
 from ..interactions.system_conf import select_driver
-from ..menu import AbstractSubMenu
-from .profile_model import ProfileConfiguration
-
-if TYPE_CHECKING:
-	_: Any
+from ..menu.abstract_menu import AbstractSubMenu
+from ..models.profile_model import ProfileConfiguration
 
 
-class ProfileMenu(AbstractSubMenu):
+class ProfileMenu(AbstractSubMenu[ProfileConfiguration]):
 	def __init__(
 		self,
-		preset: ProfileConfiguration | None = None
+		preset: ProfileConfiguration | None = None,
 	):
 		if preset:
-			self._preset = preset
+			self._profile_config = preset
 		else:
-			self._preset = ProfileConfiguration()
-
-		self._data_store: dict[str, Any] = {}
+			self._profile_config = ProfileConfiguration()
 
 		menu_optioons = self._define_menu_options()
 		self._item_group = MenuItemGroup(menu_optioons, checkmarks=True)
 
-		super().__init__(self._item_group, data_store=self._data_store, allow_reset=True)
+		super().__init__(
+			self._item_group,
+			self._profile_config,
+			allow_reset=True,
+		)
 
 	def _define_menu_options(self) -> list[MenuItem]:
 		return [
 			MenuItem(
-				text=str(_('Type')),
-				action=lambda x: self._select_profile(x),
-				value=self._preset.profile,
+				text=tr('Type'),
+				action=self._select_profile,
+				value=self._profile_config.profile,
 				preview_action=self._preview_profile,
-				key='profile'
+				key='profile',
 			),
 			MenuItem(
-				text=str(_('Graphics driver')),
-				action=lambda x: self._select_gfx_driver(x),
-				value=self._preset.gfx_driver if self._preset.profile and self._preset.profile.is_graphic_driver_supported() else None,
+				text=tr('Graphics driver'),
+				action=self._select_gfx_driver,
+				value=self._profile_config.gfx_driver if self._profile_config.profile and self._profile_config.profile.is_graphic_driver_supported() else None,
 				preview_action=self._prev_gfx,
-				enabled=self._preset.profile.is_graphic_driver_supported() if self._preset.profile else False,
+				enabled=self._profile_config.profile.is_graphic_driver_supported() if self._profile_config.profile else False,
 				dependencies=['profile'],
 				key='gfx_driver',
 			),
 			MenuItem(
-				text=str(_('Greeter')),
+				text=tr('Greeter'),
 				action=lambda x: select_greeter(preset=x),
-				value=self._preset.greeter if self._preset.profile and self._preset.profile.is_greeter_supported() else None,
-				enabled=self._preset.profile.is_graphic_driver_supported() if self._preset.profile else False,
+				value=self._profile_config.greeter if self._profile_config.profile and self._profile_config.profile.is_greeter_supported() else None,
+				enabled=self._profile_config.profile.is_graphic_driver_supported() if self._profile_config.profile else False,
 				preview_action=self._prev_greeter,
 				dependencies=['profile'],
 				key='greeter',
-			)
+			),
 		]
 
-	def run(self) -> ProfileConfiguration | None:
-		super().run()
-
-		if self._data_store.get('profile', None):
-			return ProfileConfiguration(
-				self._data_store.get('profile', None),
-				self._data_store.get('gfx_driver', None),
-				self._data_store.get('greeter', None),
-			)
-
-		return None
+	@override
+	def run(self, additional_title: str | None = None) -> ProfileConfiguration | None:
+		super().run(additional_title=additional_title)
+		return self._profile_config
 
 	def _select_profile(self, preset: Profile | None) -> Profile | None:
 		profile = select_profile(preset)
@@ -105,20 +101,20 @@ class ProfileMenu(AbstractSubMenu):
 
 			if driver and 'Sway' in profile.current_selection_names():
 				if driver.is_nvidia():
-					header = str(_('The proprietary Nvidia driver is not supported by Sway.')) + '\n'
-					header += str(_('It is likely that you will run into issues, are you okay with that?')) + '\n'
+					header = tr('The proprietary Nvidia driver is not supported by Sway.') + '\n'
+					header += tr('It is likely that you will run into issues, are you okay with that?') + '\n'
 
 					group = MenuItemGroup.yes_no()
 					group.focus_item = MenuItem.no()
 					group.default_item = MenuItem.no()
 
-					result = SelectMenu(
+					result = SelectMenu[bool](
 						group,
 						header=header,
 						allow_skip=False,
 						columns=2,
 						orientation=Orientation.HORIZONTAL,
-						alignment=Alignment.CENTER
+						alignment=Alignment.CENTER,
 					).run()
 
 					if result.item() == MenuItem.no():
@@ -135,7 +131,7 @@ class ProfileMenu(AbstractSubMenu):
 
 	def _prev_greeter(self, item: MenuItem) -> str | None:
 		if item.value:
-			return f'{_("Greeter")}: {item.value.value}'
+			return f'{tr("Greeter")}: {item.value.value}'
 		return None
 
 	def _preview_profile(self, item: MenuItem) -> str | None:
@@ -144,7 +140,7 @@ class ProfileMenu(AbstractSubMenu):
 
 		if profile:
 			if (sub_profiles := profile.current_selection) is not None:
-				text += str(_('Selected profiles: '))
+				text += tr('Selected profiles: ')
 				text += ', '.join([p.name for p in sub_profiles]) + '\n'
 
 			if packages := profile.packages_text(include_sub_packages=True):
@@ -158,7 +154,7 @@ class ProfileMenu(AbstractSubMenu):
 
 def select_greeter(
 	profile: Profile | None = None,
-	preset: GreeterType | None = None
+	preset: GreeterType | None = None,
 ) -> GreeterType | None:
 	if not profile or profile.is_greeter_supported():
 		items = [MenuItem(greeter.value, value=greeter) for greeter in GreeterType]
@@ -173,11 +169,11 @@ def select_greeter(
 
 		group.set_default_by_value(default)
 
-		result = SelectMenu(
+		result = SelectMenu[GreeterType](
 			group,
 			allow_skip=True,
-			frame=FrameProperties.min(str(_('Greeter'))),
-			alignment=Alignment.CENTER
+			frame=FrameProperties.min(tr('Greeter')),
+			alignment=Alignment.CENTER,
 		).run()
 
 		match result.type_:
@@ -197,22 +193,23 @@ def select_profile(
 	allow_reset: bool = True,
 ) -> Profile | None:
 	from archinstall.lib.profile.profiles_handler import profile_handler
+
 	top_level_profiles = profile_handler.get_top_level_profiles()
 
 	if header is None:
-		header = str(_('This is a list of pre-programmed default_profiles')) + '\n'
+		header = tr('This is a list of pre-programmed default_profiles') + '\n'
 
 	items = [MenuItem(p.name, value=p) for p in top_level_profiles]
 	group = MenuItemGroup(items, sort_items=True)
 	group.set_selected_by_value(current_profile)
 
-	result = SelectMenu(
+	result = SelectMenu[Profile](
 		group,
 		header=header,
 		allow_reset=allow_reset,
 		allow_skip=True,
 		alignment=Alignment.CENTER,
-		frame=FrameProperties.min(str(_('Main profile')))
+		frame=FrameProperties.min(tr('Main profile')),
 	).run()
 
 	match result.type_:
@@ -221,7 +218,7 @@ def select_profile(
 		case ResultType.Skip:
 			return current_profile
 		case ResultType.Selection:
-			profile_selection: Profile = result.get_value()
+			profile_selection = result.get_value()
 			select_result = profile_selection.do_on_select()
 
 			if not select_result:
@@ -240,5 +237,3 @@ def select_profile(
 					pass
 
 			return current_profile
-
-	return None
