@@ -1,45 +1,37 @@
 from __future__ import annotations
 
 import curses
-import curses.panel
-import dataclasses
 import os
 import signal
 import sys
 from abc import ABCMeta, abstractmethod
 from collections.abc import Callable
+from curses.ascii import isprint
 from curses.textpad import Textbox
-from dataclasses import dataclass
 from types import FrameType, TracebackType
-from typing import TYPE_CHECKING, Literal, override
+from typing import Literal, override
+
+from archinstall.lib.translationhandler import tr
 
-from ..lib.output import debug
 from .help import Help
-from .menu_item import MenuItem, MenuItemGroup
+from .menu_item import MenuItem, MenuItemGroup, MenuItemsState
+from .result import Result, ResultType
 from .types import (
 	SCROLL_INTERVAL,
 	STYLE,
 	Alignment,
 	Chars,
+	FrameDim,
 	FrameProperties,
 	FrameStyle,
-	MenuCell,
 	MenuKeys,
 	Orientation,
 	PreviewStyle,
-	Result,
-	ResultType,
 	ViewportEntry,
-	_FrameDim,
 )
 
-if TYPE_CHECKING:
-	from archinstall.lib.translationhandler import DeferredTranslation
 
-	_: Callable[[str], DeferredTranslation]
-
-
-class AbstractCurses(metaclass=ABCMeta):
+class AbstractCurses[ValueT](metaclass=ABCMeta):
 	def __init__(self) -> None:
 		self._help_window = self._set_help_viewport()
 
@@ -48,7 +40,7 @@ class AbstractCurses(metaclass=ABCMeta):
 		pass
 
 	@abstractmethod
-	def kickoff(self, win: 'curses._CursesWindow') -> Result:
+	def kickoff(self, win: curses.window) -> Result[ValueT]:
 		pass
 
 	def clear_all(self) -> None:
@@ -70,17 +62,17 @@ class AbstractCurses(metaclass=ABCMeta):
 			height,
 			x_start,
 			int((max_height / 2) - height / 2),
-			frame=FrameProperties.min(str(_('Archinstall help')))
+			frame=FrameProperties.min(tr('Archinstall help')),
 		)
 
 	def _confirm_interrupt(self, warning: str) -> bool:
 		while True:
-			result = SelectMenu(
+			result = SelectMenu[bool](
 				MenuItemGroup.yes_no(),
 				header=warning,
 				alignment=Alignment.CENTER,
 				columns=2,
-				orientation=Orientation.HORIZONTAL
+				orientation=Orientation.HORIZONTAL,
 			).run()
 
 			match result.type_:
@@ -90,14 +82,10 @@ class AbstractCurses(metaclass=ABCMeta):
 
 			return False
 
-	def help_entry(self) -> ViewportEntry:
-		return ViewportEntry(str(_('Press Ctrl+h for help')), 0, 0, STYLE.NORMAL)
+	def help_text(self) -> str:
+		return tr('Press Ctrl+h for help')
 
 	def _show_help(self) -> None:
-		if not self._help_window:
-			debug('no help window set')
-			return
-
 		help_text = Help.get_help_text()
 		lines = help_text.split('\n')
 
@@ -105,28 +93,21 @@ class AbstractCurses(metaclass=ABCMeta):
 		entries += [ViewportEntry(f'   {e}   ', idx + 1, 0, STYLE.NORMAL) for idx, e in enumerate(lines)]
 		self._help_window.update(entries, 0)
 
-	def get_header_entries(
-		self,
-		header: str | None,
-		offset: int = 0
-	) -> list[ViewportEntry]:
-		cur_row = 0
+	def get_header_entries(self, header: str) -> list[ViewportEntry]:
 		full_header = []
+		rows = header.split('\n')
 
-		if header:
-			for line in header.split('\n'):
-				full_header += [ViewportEntry(line, cur_row, offset, STYLE.NORMAL)]
-				cur_row += 1
+		for cur_row, line in enumerate(rows):
+			full_header += [ViewportEntry(line, cur_row, 0, STYLE.NORMAL)]
 
 		return full_header
 
 
-@dataclass
 class AbstractViewport:
 	def __init__(self) -> None:
 		pass
 
-	def add_str(self, screen: 'curses._CursesWindow', row: int, col: int, text: str, color: STYLE) -> None:
+	def add_str(self, screen: curses.window, row: int, col: int, text: str, color: STYLE) -> None:
 		try:
 			screen.addstr(row, col, text, Tui.t().get_color(color))
 		except curses.error:
@@ -139,7 +120,7 @@ class AbstractViewport:
 		max_width: int,
 		max_height: int,
 		frame: FrameProperties,
-		scroll_pct: int | None = None
+		scroll_pct: int | None = None,
 	) -> list[ViewportEntry]:
 		if not entries:
 			return []
@@ -159,13 +140,13 @@ class AbstractViewport:
 
 		# adjust the original rows and cols of the entries as
 		# they need to be shrunk by 1 to make space for the frame
-		entries = self._adjust_entries(entries, dim)
+		entries = self._adjust_entries(entries)
 
 		framed_entries = [
 			top_ve,
 			bottom_ve,
 			*frame_border,
-			*entries
+			*entries,
 		]
 
 		return framed_entries
@@ -177,8 +158,8 @@ class AbstractViewport:
 
 	def _get_right_frame(
 		self,
-		dim: _FrameDim,
-		scroll_percentage: int | None = None
+		dim: FrameDim,
+		scroll_percentage: int | None = None,
 	) -> list[ViewportEntry]:
 		right_frame = {}
 		scroll_height = int(dim.height * scroll_percentage // 100) if scroll_percentage else 0
@@ -200,10 +181,10 @@ class AbstractViewport:
 
 	def _get_top(
 		self,
-		dim: _FrameDim,
+		dim: FrameDim,
 		h_bar: str,
 		frame: FrameProperties,
-		scroll_percentage: int | None = None
+		scroll_percentage: int | None = None,
 	) -> ViewportEntry:
 		top = self._replace_str(h_bar, 1, f' {frame.header} ') if frame.header else h_bar
 
@@ -216,9 +197,9 @@ class AbstractViewport:
 
 	def _get_bottom(
 		self,
-		dim: _FrameDim,
+		dim: FrameDim,
 		h_bar: str,
-		scroll_pct: int | None = None
+		scroll_pct: int | None = None,
 	) -> ViewportEntry:
 		if scroll_pct is None:
 			bottom = Chars.Lower_left + h_bar + Chars.Lower_right
@@ -232,8 +213,8 @@ class AbstractViewport:
 		entries: list[ViewportEntry],
 		max_width: int,
 		max_height: int,
-		frame: FrameProperties
-	) -> _FrameDim:
+		frame: FrameProperties,
+	) -> FrameDim:
 		rows = self._assemble_entries(entries).split('\n')
 		header_len = len(frame.header) if frame.header else 0
 		header_len += 3  # for header padding
@@ -256,12 +237,11 @@ class AbstractViewport:
 			frame_end = max_width
 			frame_height = max_height - 1
 
-		return _FrameDim(frame_start, frame_end, frame_height)
+		return FrameDim(frame_start, frame_end, frame_height)
 
 	def _adjust_entries(
 		self,
 		entries: list[ViewportEntry],
-		frame_dim: _FrameDim
 	) -> list[ViewportEntry]:
 		for entry in entries:
 			# top row frame offset
@@ -282,7 +262,7 @@ class AbstractViewport:
 
 	def _replace_str(self, text: str, index: int = 0, replacement: str = '') -> str:
 		len_replace = len(replacement)
-		return f'{text[:index]}{replacement}{text[index + len_replace:]}'
+		return f'{text[:index]}{replacement}{text[index + len_replace :]}'
 
 	def _assemble_entries(self, entries: list[ViewportEntry]) -> str:
 		if not entries:
@@ -307,10 +287,10 @@ class EditViewport(AbstractViewport):
 		edit_height: int,
 		x_start: int,
 		y_start: int,
-		process_key: Callable[[int], int | None],
+		process_key: Callable[[int], int],
 		frame: FrameProperties,
 		alignment: Alignment = Alignment.CENTER,
-		hide_input: bool = False
+		hide_input: bool = False,
 	) -> None:
 		super().__init__()
 
@@ -321,13 +301,13 @@ class EditViewport(AbstractViewport):
 		self._edit_height = edit_height
 		self.x_start = x_start
 		self.y_start = y_start
-		self.process_key = process_key
+		self._process_key_cb = process_key
 		self._frame = frame
 		self._alignment = alignment
 		self._hide_input = hide_input
 
-		self._main_win: 'curses._CursesWindow | None' = None
-		self._edit_win: 'curses._CursesWindow | None' = None
+		self._main_win: curses.window | None = None
+		self._edit_win: curses.window | None = None
 		self._textbox: Textbox | None = None
 
 		self._init_wins()
@@ -344,7 +324,7 @@ class EditViewport(AbstractViewport):
 			1,
 			self._edit_width - 2,
 			self.y_start + 1,
-			self.x_start + x_offset + 1
+			self.x_start + x_offset + 1,
 		)
 
 	def update(self) -> None:
@@ -357,7 +337,7 @@ class EditViewport(AbstractViewport):
 			[ViewportEntry('', 0, 0, STYLE.NORMAL)],
 			self._edit_width,
 			3,
-			frame=self._frame
+			frame=self._frame,
 		)
 
 		x_offset = 0
@@ -370,11 +350,16 @@ class EditViewport(AbstractViewport):
 				row.row,
 				row.col + x_offset,
 				row.text,
-				row.style
+				row.style,
 			)
 
 		self._main_win.refresh()
 
+	def textbox_value(self) -> str:
+		if not self._textbox:
+			return ''
+		return self._textbox.gather().strip()
+
 	def erase(self) -> None:
 		if self._main_win:
 			self._main_win.erase()
@@ -391,30 +376,12 @@ class EditViewport(AbstractViewport):
 		# if this gets initialized multiple times it will be an overlay
 		# and ENTER has to be pressed multiple times to accept
 		if not self._textbox:
-			self._textbox = curses.textpad.Textbox(self._edit_win)
+			self._textbox = Textbox(self._edit_win)
 			self._main_win.refresh()
 
-		self._textbox.edit(self.process_key)  # type: ignore[arg-type]
+		self._textbox.edit(self._process_key_cb)
 
 
-@dataclass
-class ViewportState:
-	cur_pos: int
-	displayed_entries: list[ViewportEntry]
-	scroll_pct: int | None
-	scroll_pos: int | None = 0
-
-	def offset(self) -> int:
-		return min([entry.row for entry in self.displayed_entries], default=0)
-
-	def get_rows(self) -> list[int]:
-		rows = set()
-		for entry in self.displayed_entries:
-			rows.add(entry.row)
-		return list(rows)
-
-
-@dataclass
 class Viewport(AbstractViewport):
 	def __init__(
 		self,
@@ -422,9 +389,8 @@ class Viewport(AbstractViewport):
 		height: int,
 		x_start: int,
 		y_start: int,
-		enable_scroll: bool = False,
 		frame: FrameProperties | None = None,
-		alignment: Alignment = Alignment.LEFT
+		alignment: Alignment = Alignment.LEFT,
 	):
 		super().__init__()
 
@@ -432,7 +398,6 @@ class Viewport(AbstractViewport):
 		self.height = height
 		self.x_start = x_start
 		self.y_start = y_start
-		self._enable_scroll = enable_scroll
 		self._frame = frame
 		self._alignment = alignment
 
@@ -440,8 +405,6 @@ class Viewport(AbstractViewport):
 		self._main_win.nodelay(False)
 		self._main_win.standout()
 
-		self._state: ViewportState | None = None
-
 	def getch(self) -> int:
 		return self._main_win.getch()
 
@@ -451,12 +414,13 @@ class Viewport(AbstractViewport):
 
 	def update(
 		self,
-		lines: list[ViewportEntry],
+		entries: list[ViewportEntry],
 		cur_pos: int = 0,
-		scroll_pos: int | None = None
+		scroll_pos: int | None = None,
 	) -> None:
-		self._state = self._get_viewport_state(lines, cur_pos, scroll_pos)
-		visible_entries = self._adjust_entries_row(self._state.displayed_entries)
+		# self._state = self._get_viewport_state(lines, cur_pos, scroll_pos)
+		# visible_entries = self._adjust_entries_row(self._state.displayed_entries)
+		visible_entries = entries
 
 		if self._frame:
 			visible_entries = self.add_frame(
@@ -464,7 +428,7 @@ class Viewport(AbstractViewport):
 				self.width,
 				self.height,
 				frame=self._frame,
-				scroll_pct=self._state.scroll_pct
+				scroll_pct=scroll_pos,
 			)
 
 		x_offset = 0
@@ -479,181 +443,75 @@ class Viewport(AbstractViewport):
 				entry.row,
 				entry.col + x_offset,
 				entry.text,
-				entry.style
+				entry.style,
 			)
 
 		self._main_win.refresh()
 
-	def _get_available_screen_rows(self) -> int:
-		y_offset = 3 if self._frame else 0
-		return self.height - y_offset
 
-	def _calc_scroll_percent(
-		self, total: int,
-		available_rows: int,
-		scroll_pos: int
-	) -> int | None:
-		if total <= available_rows:
-			return None
-
-		percentage = int(scroll_pos / total * 100)
-
-		if percentage + SCROLL_INTERVAL > 100:
-			percentage = 100
-
-		return percentage
-
-	def _get_viewport_state(
-		self,
-		entries: list[ViewportEntry],
-		cur_pos: int,
-		scroll_pos: int | None = 0
-	) -> ViewportState:
-		if not entries:
-			return ViewportState(cur_pos, [], 0)
-
-		# we will be checking if the cursor pos is in the same window
-		# of rows as the previous selection, in that case we can keep
-		# the currently shown entries to prevent weird moving in long lists
-		if self._state is not None and scroll_pos is None:
-			rows = self._state.get_rows()
-
-			if cur_pos in rows:
-				same_row_entries = [entry for entry in entries if entry.row in rows]
-				return ViewportState(
-					cur_pos,
-					same_row_entries,
-					self._state.scroll_pct
-				)
-
-		total_rows = max([e.row for e in entries]) + 1  # rows start with 0 so add 1 for the count
-		screen_rows = self._get_available_screen_rows()
-		visible_entries = self._get_visible_entries(
-			entries,
-			cur_pos,
-			screen_rows,
-			scroll_pos,
-			total_rows
-		)
-
-		if scroll_pos is not None:
-			percentage = self._calc_scroll_percent(total_rows, screen_rows, scroll_pos)
-		else:
-			percentage = None
-
-		return ViewportState(
-			cur_pos,
-			visible_entries,
-			percentage,
-			scroll_pos
-		)
-
-	def _get_visible_entries(
-		self,
-		entries: list[ViewportEntry],
-		cur_pos: int,
-		screen_rows: int,
-		scroll_pos: int | None,
-		total_rows: int
-	) -> list[ViewportEntry]:
-		if scroll_pos is not None:
-			if total_rows <= screen_rows:
-				start = 0
-				end = total_rows
-			else:
-				start = scroll_pos
-				end = scroll_pos + screen_rows
-		else:
-			if total_rows <= screen_rows:
-				start = 0
-				end = total_rows
-			else:
-				if self._state is None:
-					if cur_pos < screen_rows:
-						start = 0
-						end = screen_rows
-					else:
-						start = cur_pos - screen_rows + 1
-						end = cur_pos + 1
-				else:
-					if cur_pos < self._state.cur_pos:
-						start = cur_pos
-						end = cur_pos + screen_rows
-					else:
-						start = cur_pos - screen_rows + 1
-						end = cur_pos + 1
-
-		return [entry for entry in entries if start <= entry.row < end]
-
-	def _adjust_entries_row(self, entries: list[ViewportEntry]) -> list[ViewportEntry]:
-		assert self._state is not None
-		modified = []
-
-		for entry in entries:
-			mod = dataclasses.replace(entry)
-			mod.row = entry.row - self._state.offset()
-			modified.append(mod)
-
-		return modified
-
-	def _unique_rows(self, entries: list[ViewportEntry]) -> int:
-		return len(set([e.row for e in entries]))
-
-
-class EditMenu(AbstractCurses):
+class EditMenu(AbstractCurses[str]):
 	def __init__(
 		self,
 		title: str,
 		edit_width: int = 50,
 		header: str | None = None,
-		validator: Callable[[str], str | None] | None = None,
+		validator: Callable[[str | None], str | None] | None = None,
 		allow_skip: bool = False,
 		allow_reset: bool = False,
 		reset_warning_msg: str | None = None,
 		alignment: Alignment = Alignment.CENTER,
 		default_text: str | None = None,
-		hide_input: bool = False
+		hide_input: bool = False,
 	):
 		super().__init__()
 
 		self._max_height, self._max_width = Tui.t().max_yx
 
 		self._header = header
+
+		self._header_entries = []
+		if header:
+			self._header_entries = self.get_header_entries(header)
+
 		self._validator = validator
 		self._allow_skip = allow_skip
 		self._allow_reset = allow_reset
 		self._interrupt_warning = reset_warning_msg
-		self._headers = self.get_header_entries(header, offset=0)
 		self._alignment = alignment
 		self._edit_width = edit_width
 		self._default_text = default_text
 		self._hide_input = hide_input
 
 		if self._interrupt_warning is None:
-			self._interrupt_warning = str(_('Are you sure you want to reset this setting?')) + '\n'
+			self._interrupt_warning = tr('Are you sure you want to reset this setting?') + '\n'
 
 		title = f'* {title}' if not self._allow_skip else title
 		self._frame = FrameProperties(title, FrameStyle.MAX)
 
-		self._help_vp: Viewport | None = None
+		self._title_vp: Viewport | None = None
 		self._header_vp: Viewport | None = None
 		self._input_vp: EditViewport | None = None
-		self._error_vp: Viewport | None = None
+		self._info_vp: Viewport | None = None
+
+		self._set_default_info = True
+		self._only_ascii_text = ViewportEntry(tr('Only ASCII characters are supported'), 0, 0, STYLE.NORMAL)
 
 		self._init_viewports()
 
-		self._last_state: Result | None = None
+		self._last_state: Result[str] | None = None
 		self._help_active = False
-		self._real_input = default_text or ""
+
+		self._current_text = default_text or ''
+		self._real_input = default_text or ''
 
 	def _init_viewports(self) -> None:
 		y_offset = 0
 
-		self._help_vp = Viewport(self._max_width, 2, 0, y_offset)
+		self._title_vp = Viewport(self._max_width, 2, 0, y_offset)
 		y_offset += 2
 
-		if self._headers:
-			header_height = len(self._headers)
+		if self._header_entries:
+			header_height = len(self._header_entries)
 			self._header_vp = Viewport(self._max_width, header_height, 0, y_offset, alignment=self._alignment)
 			y_offset += header_height
 
@@ -666,13 +524,13 @@ class EditMenu(AbstractCurses):
 			self._process_edit_key,
 			frame=self._frame,
 			alignment=self._alignment,
-			hide_input=self._hide_input
+			hide_input=self._hide_input,
 		)
 
 		y_offset += 3
-		self._error_vp = Viewport(self._max_width, 1, 0, y_offset, alignment=self._alignment)
+		self._info_vp = Viewport(self._max_width, 1, 0, y_offset, alignment=self._alignment)
 
-	def input(self) -> Result:
+	def input(self) -> Result[str]:
 		result = Tui.run(self)
 
 		assert not result.has_item() or isinstance(result.text(), str)
@@ -685,46 +543,53 @@ class EditMenu(AbstractCurses):
 		self._draw()
 
 	def _clear_all(self) -> None:
-		if self._help_vp:
-			self._help_vp.erase()
+		if self._title_vp:
+			self._title_vp.erase()
 		if self._header_vp:
 			self._header_vp.erase()
 		if self._input_vp:
 			self._input_vp.erase()
-		if self._error_vp:
-			self._error_vp.erase()
+		if self._info_vp:
+			self._info_vp.erase()
 
 	def _get_input_text(self) -> str | None:
 		assert self._input_vp
-		assert self._error_vp
+		assert self._info_vp
 
 		text = self._real_input
 
 		self.clear_all()
 
-		if text and self._validator:
+		if self._validator:
 			if (err := self._validator(text)) is not None:
 				self.clear_all()
 				entry = ViewportEntry(err, 0, 0, STYLE.ERROR)
-				self._error_vp.update([entry], 0)
+				self._info_vp.update([entry], 0)
+				self._set_default_info = False
 				self._real_input = ''
 				return None
 
 		return text
 
 	def _draw(self) -> None:
-		if self._help_vp:
-			self._help_vp.update([self.help_entry()], 0)
+		if self._title_vp:
+			help_text = self.help_text()
+			help_entry = ViewportEntry(help_text, 0, 0, STYLE.NORMAL)
+			self._title_vp.update([help_entry], 0)
 
-		if self._headers and self._header_vp:
-			self._header_vp.update(self._headers, 0)
+		if self._header_entries and self._header_vp:
+			self._header_vp.update(self._header_entries, 0)
 
 		if self._input_vp:
 			self._input_vp.update()
-			self._input_vp.edit(default_text=self._default_text)
+
+			if self._set_default_info and self._info_vp:
+				self._info_vp.update([self._only_ascii_text], 0)
+
+			self._input_vp.edit(default_text=self._current_text)
 
 	@override
-	def kickoff(self, win: 'curses._CursesWindow') -> Result:
+	def kickoff(self, win: curses.window) -> Result[str]:
 		try:
 			self._draw()
 		except KeyboardInterrupt:
@@ -749,7 +614,7 @@ class EditMenu(AbstractCurses):
 
 		return self._last_state
 
-	def _process_edit_key(self, key: int) -> int | None:
+	def _process_edit_key(self, key: int) -> int:
 		key_handles = MenuKeys.from_ord(key)
 
 		if self._help_active:
@@ -757,7 +622,7 @@ class EditMenu(AbstractCurses):
 				self._help_active = False
 				self.clear_help_win()
 				return 7
-			return None
+			return 0
 
 		# remove standard keys from the list of key handles
 		key_handles = [key for key in key_handles if key != MenuKeys.STD_KEYS]
@@ -768,14 +633,13 @@ class EditMenu(AbstractCurses):
 
 			match special_key:
 				case MenuKeys.HELP:
+					assert self._input_vp
+					self._current_text = self._input_vp.textbox_value()
 					self._clear_all()
 					self._help_active = True
 					self._show_help()
-					return None
+					return 0
 				case MenuKeys.ESC:
-					if self._help_active:
-						self._help_active = False
-						self._draw()
 					if self._allow_skip:
 						self._last_state = Result(ResultType.Skip, None)
 						key = 7
@@ -786,14 +650,16 @@ class EditMenu(AbstractCurses):
 					if len(self._real_input) > 0:
 						self._real_input = self._real_input[:-1]
 				case _:
+					if isprint(key):
+						self._real_input += chr(key)
+						if self._hide_input:
+							key = 42
+		else:
+			try:
+				if isprint(key):
 					self._real_input += chr(key)
 					if self._hide_input:
 						key = 42
-		else:
-			try:
-				self._real_input += chr(key)
-				if self._hide_input:
-					key = 42
 			except Exception:
 				pass
 
@@ -809,7 +675,7 @@ class EditMenu(AbstractCurses):
 		return True
 
 
-class SelectMenu(AbstractCurses):
+class SelectMenu[ValueT](AbstractCurses[ValueT]):
 	def __init__(
 		self,
 		group: MenuItemGroup,
@@ -828,6 +694,7 @@ class SelectMenu(AbstractCurses):
 		preview_style: PreviewStyle = PreviewStyle.NONE,
 		preview_size: float | Literal['auto'] = 0.2,
 		preview_frame: FrameProperties | None = None,
+		additional_title: str | None = None,
 	):
 		super().__init__()
 
@@ -848,26 +715,26 @@ class SelectMenu(AbstractCurses):
 		self._frame = frame
 		self._interrupt_warning = reset_warning_msg
 		self._header = header
+		self._additional_title = additional_title
 
-		header_offset = self._get_header_offset(header)
-		self._headers = self.get_header_entries(header, offset=header_offset)
+		self._header_entries = []
+		if header:
+			self._header_entries = self.get_header_entries(header)
 
 		if self._interrupt_warning is None:
-			self._interrupt_warning = str(_('Are you sure you want to reset this setting?')) + '\n'
+			self._interrupt_warning = tr('Are you sure you want to reset this setting?') + '\n'
 
 		if self._orientation == Orientation.HORIZONTAL:
 			self._horizontal_cols = columns
 		else:
 			self._horizontal_cols = 1
 
-		self._row_entries: list[list[MenuCell]] = []
 		self._prev_scroll_pos: int = 0
-		self._cur_pos: int | None = None
 
 		self._visible_entries: list[ViewportEntry] = []
 		self._max_height, self._max_width = Tui.t().max_yx
 
-		self._help_vp: Viewport | None = None
+		self._title_vp: Viewport | None = None
 		self._header_vp: Viewport | None = None
 		self._footer_vp: Viewport | None = None
 		self._menu_vp: Viewport | None = None
@@ -875,28 +742,21 @@ class SelectMenu(AbstractCurses):
 
 		self._init_viewports(preview_size)
 
-	def _get_header_offset(self, header: str | None) -> int:
-		# WARNING: any changes here will impact the list manager table view
-		if self._orientation == Orientation.HORIZONTAL:
-			return 0
+		assert self._menu_vp is not None
+		self._items_state: MenuItemsState = MenuItemsState(  # type: ignore[unreachable]
+			self._item_group,
+			total_cols=self._horizontal_cols,
+			total_rows=self._menu_vp.height,
+			with_frame=self._frame is not None,
+		)
 
-		lines = header.split('\n') if header else []
-		table_header = [line for line in lines if '|' in line]
-		longest_header = len(table_header[0]) if table_header else 0
-		longest_entry = self._item_group.max_width
-
-		delta = abs(longest_header - longest_entry)
-		offset = delta + 3  # 3 because it seems to align it...
-
-		return offset
-
-	def run(self) -> Result:
+	def run(self) -> Result[ValueT]:
 		result = Tui.run(self)
 		self._clear_all()
 		return result
 
 	@override
-	def kickoff(self, win: 'curses._CursesWindow') -> Result:
+	def kickoff(self, win: curses.window) -> Result[ValueT]:
 		self._draw()
 
 		while True:
@@ -931,8 +791,8 @@ class SelectMenu(AbstractCurses):
 			self._preview_vp.erase()
 		if self._footer_vp:
 			self._footer_vp.erase()
-		if self._help_vp:
-			self._help_vp.erase()
+		if self._title_vp:
+			self._title_vp.erase()
 
 	def _footer_entries(self) -> list[ViewportEntry]:
 		if self._active_search:
@@ -945,17 +805,17 @@ class SelectMenu(AbstractCurses):
 		footer_height = 2  # possible filter at the bottom
 		y_offset = 0
 
-		self._help_vp = Viewport(self._max_width, 2, 0, y_offset)
+		self._title_vp = Viewport(self._max_width, 2, 0, y_offset)
 		y_offset += 2
 
-		if self._headers:
-			header_height = len(self._headers)
+		if self._header_entries:
+			header_height = len(self._header_entries)
 			self._header_vp = Viewport(
 				self._max_width,
 				header_height,
 				0,
 				y_offset,
-				alignment=self._alignment
+				alignment=self._alignment,
 			)
 			y_offset += header_height
 
@@ -973,14 +833,14 @@ class SelectMenu(AbstractCurses):
 					0,
 					y_offset,
 					frame=self._frame,
-					alignment=self._alignment
+					alignment=self._alignment,
 				)
 				self._preview_vp = Viewport(
 					self._max_width,
 					prev_size,
 					0,
 					menu_height + y_offset,
-					frame=self._preview_frame
+					frame=self._preview_frame,
 				)
 			case PreviewStyle.RIGHT:
 				menu_width = self._max_width - prev_size
@@ -991,7 +851,7 @@ class SelectMenu(AbstractCurses):
 					0,
 					y_offset,
 					frame=self._frame,
-					alignment=self._alignment
+					alignment=self._alignment,
 				)
 				self._preview_vp = Viewport(
 					prev_size,
@@ -999,7 +859,7 @@ class SelectMenu(AbstractCurses):
 					menu_width,
 					y_offset,
 					frame=self._preview_frame,
-					alignment=self._alignment
+					alignment=self._alignment,
 				)
 			case PreviewStyle.TOP:
 				menu_height = available_height - prev_size
@@ -1010,7 +870,7 @@ class SelectMenu(AbstractCurses):
 					0,
 					prev_size + y_offset,
 					frame=self._frame,
-					alignment=self._alignment
+					alignment=self._alignment,
 				)
 				self._preview_vp = Viewport(
 					self._max_width,
@@ -1018,7 +878,7 @@ class SelectMenu(AbstractCurses):
 					0,
 					y_offset,
 					frame=self._preview_frame,
-					alignment=self._alignment
+					alignment=self._alignment,
 				)
 			case PreviewStyle.NONE:
 				self._menu_vp = Viewport(
@@ -1027,20 +887,20 @@ class SelectMenu(AbstractCurses):
 					0,
 					y_offset,
 					frame=self._frame,
-					alignment=self._alignment
+					alignment=self._alignment,
 				)
 
 		self._footer_vp = Viewport(
 			self._max_width,
 			footer_height,
 			0,
-			self._max_height - footer_height
+			self._max_height - footer_height,
 		)
 
 	def _determine_prev_size(
 		self,
 		preview_size: float | Literal['auto'],
-		offset: int = 0
+		offset: int = 0,
 	) -> int:
 		if not isinstance(preview_size, float) and preview_size != 'auto':
 			raise ValueError('preview size must be a float or "auto"')
@@ -1050,7 +910,7 @@ class SelectMenu(AbstractCurses):
 		if preview_size == 'auto':
 			match self._preview_style:
 				case PreviewStyle.RIGHT:
-					menu_width = self._item_group.max_width + 5
+					menu_width = self._item_group.get_max_width() + 5
 					if self._multi:
 						menu_width += 5
 					prev_size = self._max_width - menu_width
@@ -1074,21 +934,24 @@ class SelectMenu(AbstractCurses):
 	def _draw(self) -> None:
 		footer_entries = self._footer_entries()
 
-		vp_entries = self._get_row_entries()
-		self._cur_pos = self._get_cursor_pos()
+		items = self._items_state.get_view_items()
+		vp_entries = self._item_to_vp_entry(items)
 
-		if self._help_vp:
-			self._update_viewport(self._help_vp, [self.help_entry()])
+		if self._title_vp:
+			title_text = self.help_text()
+
+			if self._additional_title is not None:
+				title_text += f' {self._additional_title}'
+
+			title_vp_entry = ViewportEntry(title_text, 0, 0, STYLE.NORMAL)
+
+			self._update_viewport(self._title_vp, [title_vp_entry])
 
 		if self._header_vp:
-			self._update_viewport(self._header_vp, self._headers)
+			self._update_viewport(self._header_vp, self._header_entries)
 
 		if self._menu_vp:
-			self._update_viewport(
-				self._menu_vp,
-				vp_entries,
-				cur_pos=self._cur_pos
-			)
+			self._update_viewport(self._menu_vp, vp_entries)
 
 		if vp_entries:
 			self._update_preview()
@@ -1102,28 +965,15 @@ class SelectMenu(AbstractCurses):
 		self,
 		viewport: Viewport,
 		entries: list[ViewportEntry],
-		cur_pos: int = 0
+		cur_pos: int = 0,
 	) -> None:
 		if entries:
 			viewport.update(entries, cur_pos=cur_pos)
 		else:
 			viewport.update([])
 
-	def _get_cursor_pos(self) -> int:
-		for idx, cells in enumerate(self._row_entries):
-			for cell in cells:
-				if self._item_group.focus_item == cell.item:
-					return idx
-		return 0
-
-	def _get_visible_items(self) -> list[MenuItem]:
-		return [it for it in self._item_group.items if self._item_group.should_enable_item(it)]
-
-	def _list_to_cols(self, items: list[MenuItem], cols: int) -> list[list[MenuItem]]:
-		return [items[i:i + cols] for i in range(0, len(items), cols)]
-
-	def _get_col_widths(self) -> list[int]:
-		cols_widths = self._calc_col_widths(self._row_entries, self._horizontal_cols)
+	def _get_col_widths(self, items: list[list[MenuItem]]) -> list[int]:
+		cols_widths = self._calc_col_widths(items, self._horizontal_cols)
 		return [col_width + len(self._cursor_char) + self._item_distance() for col_width in cols_widths]
 
 	def _item_distance(self) -> int:
@@ -1132,68 +982,56 @@ class SelectMenu(AbstractCurses):
 		else:
 			return self._column_spacing
 
-	def _get_row_entries(self) -> list[ViewportEntry]:
-		cells = self._assemble_menu_cells()
+	def _item_to_vp_entry(self, items: list[list[MenuItem]]) -> list[ViewportEntry]:
 		entries = []
+		cols_widths = self._get_col_widths(items)
 
-		self._row_entries = [cells[x:x + self._horizontal_cols] for x in range(0, len(cells), self._horizontal_cols)]
-		cols_widths = self._get_col_widths()
-
-		for row_idx, row in enumerate(self._row_entries):
+		for row_idx, row in enumerate(items):
 			cur_pos = len(self._cursor_char)
 
 			for col_idx, cell in enumerate(row):
 				cur_text = ''
 				style = STYLE.NORMAL
 
-				if cell.item == self._item_group.focus_item:
+				if cell == self._item_group.focus_item:
 					cur_text = self._cursor_char
 					style = STYLE.MENU_STYLE
 
 				entries += [ViewportEntry(cur_text, row_idx, cur_pos - len(self._cursor_char), STYLE.CURSOR_STYLE)]
 
-				entries += [ViewportEntry(cell.text, row_idx, cur_pos, style)]
-				cur_pos += len(cell.text)
+				menu_item_text = self._menu_item_text(cell)
+				entries += [ViewportEntry(menu_item_text, row_idx, cur_pos, style)]
+				cur_pos += len(menu_item_text)
 
 				if col_idx < len(row) - 1:
-					spacer_len = cols_widths[col_idx] - len(cell.text)
+					spacer_len = cols_widths[col_idx] - len(menu_item_text)
 					entries += [ViewportEntry(' ' * spacer_len, row_idx, cur_pos, STYLE.NORMAL)]
 					cur_pos += spacer_len
 
 		return entries
 
-	def _calc_col_widths(
-		self,
-		row_chunks: list[list[MenuCell]],
-		cols: int
-	) -> list[int]:
+	def _calc_col_widths(self, rows: list[list[MenuItem]], columns: int) -> list[int]:
 		col_widths = []
-		for col in range(cols):
+
+		for row in rows:
 			col_entries = []
-			for row in row_chunks:
-				if col < len(row):
-					col_entries += [len(row[col].text)]
+			for column in range(columns):
+				if column < len(row):
+					col_entries += [len(row[column].text)]
 
 			if col_entries:
-				col_widths += [max(col_entries) if col_entries else 0]
+				col_widths += [max(col_entries)]
 
 		return col_widths
 
-	def _assemble_menu_cells(self) -> list[MenuCell]:
-		items = self._get_visible_items()
-		entries = []
+	def _menu_item_text(self, item: MenuItem) -> str:
+		item_text = ''
 
-		for item in items:
-			item_text = ''
+		if self._multi and not item.is_empty():
+			item_text += self._multi_prefix(item)
 
-			if self._multi and not item.is_empty():
-				item_text += self._multi_prefix(item)
-
-			item_text += self._item_group.get_item_text(item)
-
-			entries += [MenuCell(item, item_text)]
-
-		return entries
+		item_text += self._item_group.get_item_text(item)
+		return item_text
 
 	def _update_preview(self) -> None:
 		if not self._preview_vp:
@@ -1214,20 +1052,75 @@ class SelectMenu(AbstractCurses):
 		preview_text = action_text.split('\n')
 		entries = [ViewportEntry(e, idx, 0, STYLE.NORMAL) for idx, e in enumerate(preview_text)]
 
-		self._calc_prev_scroll_pos(entries)
+		total_prev_rows = max([e.row for e in entries]) + 1  # rows start with 0 and we need the count
+		available_rows = self._preview_vp.height - 2  # for the preview frame
 
-		self._preview_vp.update(entries, scroll_pos=self._prev_scroll_pos)
+		self._calc_prev_scroll_pos(entries, total_prev_rows)
+		prev_entries = self._get_scroll_win_prev_entries(entries, total_prev_rows, available_rows)
+		scroll_pct = self._get_scroll_pct(total_prev_rows, available_rows)
 
-	def _calc_prev_scroll_pos(self, entries: list[ViewportEntry]) -> None:
-		total_rows = max([e.row for e in entries]) + 1  # rows start with 0 and we need the count
+		self._preview_vp.update(prev_entries, scroll_pos=scroll_pct)
 
-		if self._prev_scroll_pos >= total_rows:
-			self._prev_scroll_pos = total_rows - 2
+	def _get_scroll_pct(
+		self,
+		total_prev_rows: int,
+		available_rows: int,
+	) -> int | None:
+		assert self._preview_vp is not None
+
+		if total_prev_rows <= available_rows:
+			return None
+
+		pct = int(self._prev_scroll_pos / total_prev_rows * 100)
+
+		if pct + SCROLL_INTERVAL > 100:
+			pct = 100
+
+		if pct < 0:
+			pct = 0
+
+		return pct
+
+	def _get_scroll_win_prev_entries(
+		self,
+		entries: list[ViewportEntry],
+		total_prev_rows: int,
+		available_rows: int,
+	) -> list[ViewportEntry]:
+		assert self._preview_vp is not None
+
+		if total_prev_rows <= available_rows:
+			start_row = 0
+		else:
+			start_row = self._prev_scroll_pos
+
+		end_row = start_row + available_rows
+
+		if end_row > total_prev_rows:
+			end_row = total_prev_rows
+
+		prev_entries = [e for e in entries if start_row <= e.row < end_row]
+
+		# normalize the rows
+		for e in prev_entries:
+			e.row -= start_row
+
+		return prev_entries
+
+	def _calc_prev_scroll_pos(
+		self,
+		entries: list[ViewportEntry],
+		total_prev_rows: int,
+	) -> None:
+		if self._prev_scroll_pos >= total_prev_rows:
+			self._prev_scroll_pos = total_prev_rows - 2
 		elif self._prev_scroll_pos < 0:
 			self._prev_scroll_pos = 0
 
 	def _multi_prefix(self, item: MenuItem) -> str:
-		if self._item_group.is_item_selected(item):
+		if item.read_only:
+			return '    '
+		elif self._item_group.is_item_selected(item):
 			return '[x] '
 		else:
 			return '[ ] '
@@ -1238,7 +1131,7 @@ class SelectMenu(AbstractCurses):
 		else:
 			return False
 
-	def _process_input_key(self, key: int) -> Result | None:
+	def _process_input_key(self, key: int) -> Result[ValueT] | None:
 		key_handles = MenuKeys.from_ord(key)
 
 		if self._help_active:
@@ -1294,12 +1187,14 @@ class SelectMenu(AbstractCurses):
 							return Result(ResultType.Selection, self._item_group.focus_item)
 
 					return None
-			case MenuKeys.MENU_UP | MenuKeys.MENU_DOWN | MenuKeys.MENU_LEFT | MenuKeys.MENU_RIGHT:
-				self._focus_item(handle)
+			case MenuKeys.MENU_DOWN | MenuKeys.MENU_RIGHT:
+				self._focus_item('next')
+			case MenuKeys.MENU_UP | MenuKeys.MENU_LEFT:
+				self._focus_item('prev')
 			case MenuKeys.MENU_START:
-				self._item_group.focus_first()
+				self._focus_item('first')
 			case MenuKeys.MENU_END:
-				self._item_group.focus_last()
+				self._focus_item('last')
 			case MenuKeys.MULTI_SELECT:
 				if self._multi:
 					self._item_group.select_current_item()
@@ -1315,7 +1210,7 @@ class SelectMenu(AbstractCurses):
 					if self._allow_skip:
 						return Result(ResultType.Skip, None)
 			case MenuKeys.NUM_KEYS:
-				self._item_group.set_focus_item_index(key - 49)
+				self._item_group.focus_index(key - 49)
 			case MenuKeys.SCROLL_DOWN:
 				self._prev_scroll_pos += SCROLL_INTERVAL
 			case MenuKeys.SCROLL_UP:
@@ -1323,56 +1218,22 @@ class SelectMenu(AbstractCurses):
 			case _:
 				pass
 
-		self._draw()
 		return None
 
-	def _focus_item(self, key: MenuKeys) -> None:
-		focus_item = self._item_group.focus_item
-		next_row = 0
-		next_col = 0
+	def _focus_item(self, direction: Literal['next', 'prev', 'first', 'last']) -> None:
+		# reset the preview scroll as the newly focused item
+		# may have a different preview row count and it'll blow up
+		self._prev_scroll_pos = 0
 
-		for row_idx, row in enumerate(self._row_entries):
-			for col_idx, cell in enumerate(row):
-				if cell.item == focus_item:
-					match key:
-						case MenuKeys.MENU_UP:
-							next_row = row_idx - 1
-							next_col = col_idx
-
-							if next_row < 0:
-								next_row = len(self._row_entries) - 1
-							if next_col >= len(self._row_entries[next_row]):
-								next_col = len(self._row_entries[next_row]) - 1
-						case MenuKeys.MENU_DOWN:
-							next_row = row_idx + 1
-							next_col = col_idx
-
-							if next_row >= len(self._row_entries):
-								next_row = 0
-							if next_col >= len(self._row_entries[next_row]):
-								next_col = len(self._row_entries[next_row]) - 1
-						case MenuKeys.MENU_RIGHT:
-							next_col = col_idx + 1
-							next_row = row_idx
-
-							if next_col >= len(self._row_entries[row_idx]):
-								next_col = 0
-								next_row = 0 if next_row == (len(self._row_entries) - 1) else next_row + 1
-						case MenuKeys.MENU_LEFT:
-							next_col = col_idx - 1
-							next_row = row_idx
-
-							if next_col < 0:
-								next_row = len(self._row_entries) - 1 if next_row == 0 else next_row - 1
-								next_col = len(self._row_entries[next_row]) - 1
-
-		if next_row < len(self._row_entries):
-			row = self._row_entries[next_row]
-			if next_col < len(row):
-				self._item_group.focus_item = row[next_col].item
-
-		if self._item_group.focus_item and self._item_group.focus_item.is_empty():
-			self._focus_item(key)
+		match direction:
+			case 'next':
+				self._item_group.focus_next()
+			case 'prev':
+				self._item_group.focus_prev()
+			case 'first':
+				self._item_group.focus_first()
+			case 'last':
+				self._item_group.focus_last()
 
 
 class Tui:
@@ -1383,11 +1244,11 @@ class Tui:
 			tui = self.init()
 			Tui._t = tui
 
-	def __exit__(self, exc_type: type[BaseException] | None, exc_val: BaseException | None, tb: TracebackType | None) -> None:
+	def __exit__(self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None) -> None:
 		self.stop()
 
 	@property
-	def screen(self) -> 'curses._CursesWindow':
+	def screen(self) -> curses.window:
 		return self._screen
 
 	@staticmethod
@@ -1444,13 +1305,13 @@ class Tui:
 		text: str,
 		row: int = 0,
 		col: int = 0,
-		endl: str | None = '\n',
-		clear_screen: bool = False
+		endl: str = '\n',
+		clear_screen: bool = False,
 	) -> None:
-		if Tui._t is None:
-			if clear_screen:
-				os.system('clear')
+		if clear_screen:
+			os.system('clear')
 
+		if Tui._t is None:
 			print(text, end=endl)
 			sys.stdout.flush()
 
@@ -1472,7 +1333,7 @@ class Tui:
 		return self._screen.getmaxyx()
 
 	@staticmethod
-	def run(component: AbstractCurses) -> Result:
+	def run[ValueT](component: AbstractCurses[ValueT]) -> Result[ValueT]:
 		if Tui._t is None:
 			tui = Tui().init()
 			tui.screen.clear()
@@ -1488,12 +1349,12 @@ class Tui:
 		if hasattr(self, '_component') and self._component is not None:  # pylint: disable=no-member
 			self._component.resize_win()  # pylint: disable=no-member
 
-	def _main_loop(self, component: AbstractCurses) -> Result:
+	def _main_loop[ValueT](self, component: AbstractCurses[ValueT]) -> Result[ValueT]:
 		self._screen.refresh()
 		return component.kickoff(self._screen)
 
 	def _reset_terminal(self) -> None:
-		os.system("reset")
+		os.system('reset')
 
 	def _set_up_colors(self) -> None:
 		curses.init_pair(STYLE.NORMAL.value, curses.COLOR_WHITE, curses.COLOR_BLACK)
